import { writeFile } from 'fs/promises'
import process from 'process'
import { basename } from 'path'

interface AnnounceOpts {
  /** Output filename */
  out: string
  releaseDir: string
  apkgFilenames: string[]
}

export async function announce (opts: AnnounceOpts): Promise<void> {
  const version = getReleaseVersion(opts.releaseDir)
  const announcement = [
    `# Sinologie Anki Pack ${version} ist released! 🎉`,
    `## Details zum Release:\n${linkToReleaseDetails(version)}`,
    `## Download komplettes Pack als ZIP:\n${
      allArtifactsZipDownload()
    }`,
    '## Download einzelne APKGs:',
    ...(await linksToApkgs(opts.releaseDir, opts.apkgFilenames))
  ].join('\n\n')
  await writeFile(opts.out, announcement, { encoding: 'utf-8' })
}

function getReleaseVersion (dir: string): string {
  const dashIdx = dir.lastIndexOf('-')
  if (dashIdx < 0) {
    throw new Error(`${dir} does not look like a release dir`)
  }
  return dir.substring(dashIdx + 1)
}

function linkToReleaseDetails (version: string): string {
  return (
    `https://gitlab.phaidra.org/kartenaale/sinologie-anki-pack/-/releases/${
      version
    }`
  )
}

async function linksToApkgs (
  releaseDir: string,
  apkgFilenames: string[]
): Promise<string[]> {
  const apkgPaths = (
    !(apkgFilenames[0]?.startsWith(`${releaseDir}/`))
  )
    ? apkgFilenames.map(p => `${releaseDir}/${p}`)
    : [...apkgFilenames]
  return apkgPaths.sort().map(path => {
    return `### ${
      basename(path, '.apkg').replace(/-/g, ' ').replace('fuer', 'für')
    }\n${
        rawArtifactDownload(path)
      }`
  })
}

function rawArtifactDownload (path: string): string {
  if (path === '') {
    throw new Error('path required')
  }
  return `https://gitlab.phaidra.org/kartenaale/sinologie-anki-pack/-/jobs/${
    getCiJobId()
  }/artifacts/raw/${encodeURIComponent(path)}`
}

function allArtifactsZipDownload (): string {
  return `https://gitlab.phaidra.org/kartenaale/sinologie-anki-pack/-/jobs/${
    getCiJobId()
  }/artifacts/download`
}

function getCiJobId (): string {
  return process.env.CI_JOB_ID ?? 'job-id-unavailable'
}
